package tsuteto.spelunker.eventhandler;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.server.MinecraftServer;
import tsuteto.spelunker.SpelunkerMod;
import tsuteto.spelunker.constants.SpelunkerPacketType;
import tsuteto.spelunker.packet.SpelunkerPacketDispatcher;
import tsuteto.spelunker.player.SpelunkerPlayerMP;
import tsuteto.spelunker.player.SpelunkerPlayerSP;
import tsuteto.spelunker.potion.SpelunkerPotion;
import cpw.mods.fml.common.ICraftingHandler;
import cpw.mods.fml.common.IPlayerTracker;

/**
 * Handles game events by FML
 *
 * @author Tsuteto
 *
 */
public class FmlGameHandler implements ICraftingHandler, IPlayerTracker
{
    @Override
    public void onCrafting(EntityPlayer player, ItemStack item, IInventory craftMatrix)
    {
    }

    @Override
    public void onSmelting(EntityPlayer player, ItemStack item)
    {
        if (!player.worldObj.isRemote)
        {
            SpelunkerPlayerMP spelunker = SpelunkerMod.getSpelunkerPlayer(player);
            if (spelunker != null)
            {
                int itemId = item.getItem().itemID;
                //System.out.println("takenFromFurnace: " + item.stackSize);

                // Obtained gold
                if (itemId == Item.ingotGold.itemID) {
                    // Passes twice when shift-clicking
                    spelunker.addSpelunkerScore(item.stackSize == 0 ? 1500 : 3000 * item.stackSize);
                }
            }
        }
    }

    @Override
    public void onPlayerLogin(EntityPlayer player)
    {
        //SpelunkerMod.sidedProxy.showSoundNotice(player);
        new SpelunkerPacketDispatcher(SpelunkerPacketType.CHECK_POTION_ID)
                .addInt(SpelunkerPotion.choked.id)
                .addInt(SpelunkerPotion.heatStroke.id)
                .sendPacketPlayer(player);
    }

    @Override
    public void onPlayerLogout(EntityPlayer player)
    {
        if (player.worldObj.isRemote)
        {
            SpelunkerPlayerSP spelunker = SpelunkerMod.getSpelunkerPlayer(player);
            if (spelunker != null)
            {
                spelunker.isReady = false;
                spelunker.isInitializing = false;
            }
        }
    }

    @Override
    public void onPlayerChangedDimension(EntityPlayer player)
    {
        // TODO Auto-generated method stub

    }

    @Override
    public void onPlayerRespawn(EntityPlayer player)
    {
        if (!player.worldObj.isRemote)
        {
            SpelunkerPlayerMP spelunker = SpelunkerMod.getSpelunkerPlayer(player);

            if (!MinecraftServer.getServer().isSinglePlayer() && SpelunkerMod.isGameToBeFinished())
            {
                spelunker.banByGameover();
            }

            if (spelunker != null)
            {
                EntityPlayerMP deadPlayer = SpelunkerMod.deadPlayerStorage.get(player.username);

                if (deadPlayer != null)
                {
                    spelunker.onRespawn(deadPlayer);
                    SpelunkerMod.deadPlayerStorage.remove(player.username);
                }
            }
        }
    }

}
