package tsuteto.spelunker.eventhandler;

import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.EntityOwnable;
import net.minecraft.entity.passive.EntityBat;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.entity.player.InventoryPlayer;
import net.minecraft.item.ItemStack;
import net.minecraftforge.event.ForgeSubscribe;
import net.minecraftforge.event.entity.living.LivingDeathEvent;
import net.minecraftforge.event.entity.living.LivingEvent;
import tsuteto.spelunker.BatDroppingsHandler;
import tsuteto.spelunker.SpelunkerMod;
import tsuteto.spelunker.damage.SpelunkerDamageSource;
import tsuteto.spelunker.item.ItemHelmet;
import tsuteto.spelunker.item.SpelunkerItem;
import tsuteto.spelunker.player.SpelunkerPlayerMP;

/**
 * Handles events on EntityLiving
 *
 * @author Tsuteto
 *
 */
public class LivingEventHandler
{
    private BatDroppingsHandler controller = BatDroppingsHandler.getInstance();

    @ForgeSubscribe
    public void onLivingUpdate(LivingEvent.LivingUpdateEvent event)
    {
        EntityLivingBase entity = event.entityLiving;

        if (entity instanceof EntityBat)
        {
            controller.onEntityUpdate((EntityBat)event.entityLiving);
        }

        int helmetInterval = entity.worldObj.isRemote ?
                SpelunkerMod.settings().clientHelmetLightInterval
                : SpelunkerMod.settings().serverHelmetLightInterval;

        if (entity.ticksExisted % helmetInterval == 0)
        {
            boolean isHelmetLighting = false;

            if (entity instanceof EntityPlayer)
            {
                EntityPlayer player = (EntityPlayer)event.entityLiving;
                InventoryPlayer var5 = player.inventory;
                ItemStack var6 = var5.armorItemInSlot(3);

                if (var6 != null && var6.itemID == SpelunkerItem.itemHelmet.itemID)
                {
                    isHelmetLighting = true;
                }
            }
            else
            {
                ItemStack var6 = entity.getCurrentItemOrArmor(1);

                if (var6 != null && var6.itemID == SpelunkerItem.itemHelmet.itemID)
                {
                    isHelmetLighting = true;
                }
            }

            if (isHelmetLighting)
            {
                ((ItemHelmet)SpelunkerItem.itemHelmet).lightTarget(entity.worldObj, entity);
            }
            else
            {
                ((ItemHelmet)SpelunkerItem.itemHelmet).removeLight(entity.worldObj, entity);
            }
        }
    }

    @ForgeSubscribe
    public void onLivingDeath(LivingDeathEvent event)
    {
        EntityLivingBase living = event.entityLiving;

        if (!living.worldObj.isRemote)
        {
            // Pet killed
            if (living instanceof EntityOwnable)
            {
                EntityOwnable tameable = (EntityOwnable)living;
                Entity owner = tameable.getOwner();
                if (owner instanceof EntityPlayerMP)
                {
                    SpelunkerPlayerMP spelunker = SpelunkerMod.getSpelunkerPlayer(((EntityPlayer)owner));
                    if (spelunker != null && spelunker.isHardcore())
                    {
                        spelunker.killInstantly(SpelunkerDamageSource.causeDamageOfPetKilled(living));
                    }
                }
            }
        }
    }
}
