package tsuteto.spelunker.item;

import java.util.ArrayList;

import net.minecraft.block.Block;
import net.minecraft.block.BlockDispenser;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.item.EnumArmorMaterial;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.src.ModLoader;
import net.minecraft.world.World;
import tsuteto.spelunker.Settings;
import tsuteto.spelunker.SpelunkerMod;
import tsuteto.spelunker.constants.SpelunkerDifficulty;
import tsuteto.spelunker.player.SpelunkerPlayerMP;

public abstract class SpelunkerItem extends Item
{
    public static Item itemEnergy;
    public static Item itemGunWood;
    public static Item itemGunSteel;
    public static Item itemGunSpelunker;
    public static Item itemSpeedPotion;
    public static Item itemDollar;
    public static Item item1up;
    public static Item itemCoin;
    public static Item itemMiracle;
    public static Item item2xScore;
    public static Item itemInvincible;
    public static Item itemDynamite;
    public static Item itemFlashDrop;
    public static Item itemHelmet;
    public static Item itemFlash;
    public static Item itemGoldenStatue;

    public static ArrayList<SpelunkerItem> dropItemRegistry = new ArrayList<SpelunkerItem>();

    public static void load()
    {
        Settings settings = SpelunkerMod.settings();

        itemGunWood = new ItemGun(settings.woodengunId, EnumGunMaterial.WOOD)
                .setUnlocalizedName("spelunker:woodenGun").setFull3D();
        itemGunSteel = new ItemGun(settings.steelgunId, EnumGunMaterial.IRON)
                .setUnlocalizedName("spelunker:steelGun").setFull3D();
        itemGunSpelunker = new ItemGun(settings.spelunkergunId, EnumGunMaterial.ORIGINAL)
                .setUnlocalizedName("spelunker:spelunkerGun").setFull3D();

        itemEnergy = new ItemEnergy(settings.energyId)
                .setUnlocalizedName("spelunker:energy");
        itemSpeedPotion = new ItemSpeedPotion(settings.speedPotionId)
                .setUnlocalizedName("spelunker:speedPotion");
        itemDollar = new ItemDoller(settings.itemDollarId)
                .setUnlocalizedName("spelunker:dollarBag");
        item1up = new Item1up(settings.itemLifeId)
                .setUnlocalizedName("spelunker:1up");
        itemCoin = new ItemCoin(settings.itemCoinId)
                .setUnlocalizedName("spelunker:coin");
        itemMiracle = new ItemMiracle(settings.itemMiracleId)
                .setUnlocalizedName("spelunker:miracle");
        item2xScore = new Item2xScore(settings.item2xId)
                .setUnlocalizedName("spelunker:2xScore");
        itemInvincible = new ItemInvincible(settings.itemInvincibleId)
                .setUnlocalizedName("spelunker:invincible");
        itemDynamite = new ItemDynamite(settings.itemDynamiteId)
                .setUnlocalizedName("spelunker:dynamite");
        itemFlashDrop = new ItemFlashDrop(settings.itemFlashDropId)
                .setUnlocalizedName("spelunker:flashDrop");

        itemFlash = new ItemFlash(settings.itemFlashId)
                .setUnlocalizedName("spelunker:flash").func_111206_d("spelunker:flash")
                .setCreativeTab(CreativeTabs.tabTools);
        itemHelmet = new ItemHelmet(settings.itemHelmetId, EnumArmorMaterial.IRON, 2, 0)
                .setLightRange(32)
                .setTargetLightValue(14)
                .setHeadLightValue(8)
                .setUnlocalizedName("spelunker:helmet").func_111206_d("spelunker:helmet")
                .setCreativeTab(CreativeTabs.tabTools);

        itemGoldenStatue = new ItemGoldenSpelunker(settings.itemGoldenStatueId)
                .setMaxStackSize(1)
                .setCreativeTab(CreativeTabs.tabMisc)
                .setUnlocalizedName("spelunker:goldenStatueF")
                .func_111206_d("spelunker:goldenSpelunker");

        // Register dispense behavior
        BlockDispenser.dispenseBehaviorRegistry.putObject(itemFlash, new BehaviorDispenseFlash());

        ModLoader.addName(itemGunWood, "en_US", "Air Blaster");
        ModLoader.addName(itemGunWood, "ja_JP", "エアブラスター");
        ModLoader.addName(itemGunSteel, "en_US", "Machine Blaster");
        ModLoader.addName(itemGunSteel, "ja_JP", "マシンブラスター");
        ModLoader.addName(itemGunSpelunker, "en_US", "Spelunker's Blaster");
        ModLoader.addName(itemGunSpelunker, "ja_JP", "スペランカーブラスター");

        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".gunBullet.name", "en_US", "Blaster Bullet");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".gunBullet.name", "ja_JP", "ブラスターの弾");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".batDroppings.name", "en_US", "Bat Droppings");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".batDroppings.name", "ja_JP", "コウモリのフン");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".flashBullet.name", "en_US", "Luminary");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".flashBullet.name", "ja_JP", "発光体");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".flash.name", "en_US", "Flash");
        ModLoader.addLocalization("entity." + SpelunkerMod.modId + ".flash.name", "ja_JP", "フラッシュ");

        ModLoader.addLocalization("entity.gunBullet.name", "en_US", "Blaster Bullet");
        ModLoader.addLocalization("entity.gunBullet.name", "ja_JP", "ブラスターの弾");
        ModLoader.addLocalization("entity.batDroppings.name", "en_US", "Bat Droppings");
        ModLoader.addLocalization("entity.batDroppings.name", "ja_JP", "コウモリのフン");
        ModLoader.addLocalization("entity.flashBullet.name", "en_US", "Luminary");
        ModLoader.addLocalization("entity.flashBullet.name", "ja_JP", "発光体");
        ModLoader.addLocalization("entity.flash.name", "en_US", "Flash");
        ModLoader.addLocalization("entity.flash.name", "ja_JP", "フラッシュ");

        ModLoader.addName(itemEnergy, "en_US", "Energy");
        ModLoader.addName(itemEnergy, "ja_JP", "エネルギー");
        ModLoader.addName(itemSpeedPotion, "en_US", "Red Potion");
        ModLoader.addName(itemSpeedPotion, "ja_JP", "赤い薬");
        ModLoader.addName(itemDollar, "en_US", "Sack of Money");
        ModLoader.addName(itemDollar, "ja_JP", "ドル袋");
        ModLoader.addName(item1up, "en_US", "1-UP");
        ModLoader.addName(item1up, "ja_JP", "1UP");
        ModLoader.addName(itemCoin, "en_US", "Coin");
        ModLoader.addName(itemCoin, "ja_JP", "コイン");
        ModLoader.addName(itemMiracle, "en_US", "Miracle");
        ModLoader.addName(itemMiracle, "ja_JP", "ミラクル");
        ModLoader.addName(item2xScore, "en_US", "2x Score");
        ModLoader.addName(item2xScore, "ja_JP", "スコア2倍");
        ModLoader.addName(itemInvincible, "en_US", "Invincible");
        ModLoader.addName(itemInvincible, "ja_JP", "無敵");
        ModLoader.addName(itemDynamite, "en_US", "Dynamite");
        ModLoader.addName(itemDynamite, "ja_JP", "ダイナマイト");
        ModLoader.addName(itemHelmet, "en_US", "Spelunker's Helmet");
        ModLoader.addName(itemHelmet, "ja_JP", "スペランカーヘルメット");
        ModLoader.addName(itemFlash, "en_US", "Flash");
        ModLoader.addName(itemFlash, "ja_JP", "フラッシュ");
        ModLoader.addName(itemFlashDrop, "en_US", "Flash");
        ModLoader.addName(itemFlashDrop, "ja_JP", "フラッシュ");

        ModLoader.addName(itemGoldenStatue, "en_US", "Golden Spelunker Statue (Fake)");
        ModLoader.addName(itemGoldenStatue, "ja_JP", "金のスペランカー像(偽)");

        // Recipes
        ModLoader.addRecipe(new ItemStack(itemGunWood),
                new Object[] {
                    "XXX",
                    "  X",
                    Character.valueOf('X'),
                    Block.planks, });
        ModLoader.addRecipe(new ItemStack(itemGunSteel),
                new Object[] {
                    "XX#",
                    "  Y",
                    Character.valueOf('X'), Item.ingotIron,
                    Character.valueOf('Y'), Block.planks,
                    Character.valueOf('#'), new ItemStack(Item.dyePowder, 1, 4) });
        ModLoader.addRecipe(new ItemStack(itemGunSpelunker),
                new Object[] {
                    "XX#",
                    "  /",
                    Character.valueOf('X'), Item.ingotIron,
                    Character.valueOf('/'), Item.blazeRod,
                    Character.valueOf('#'), Item.enderPearl });

        ModLoader.addRecipe(new ItemStack(itemHelmet, 1),
                new Object[] {
                    "HCG",
                    'H', Item.helmetIron,
                    'C', new ItemStack(Item.coal, 1, 0),
                    'G', Block.thinGlass});

        ModLoader.addRecipe(new ItemStack(itemFlash, 8),
                new Object[] {
                    "L",
                    "G",
                    "B",
                    'L', Item.glowstone,
                    'G', Item.gunpowder,
                    'B', Item.glassBottle});

        ModLoader.addRecipe(new ItemStack(itemFlash, 8),
                new Object[] {
                    "G",
                    "L",
                    "B",
                    'G', Item.gunpowder,
                    'L', Item.glowstone,
                    'B', Item.glassBottle});

        if (settings.goldenSpelunkerRecipe)
        {
            ModLoader.addShapelessRecipe(new ItemStack(itemGoldenStatue),
                    new Object[] {
                        new ItemStack(Item.dyePowder, 1, 11),
                        new ItemStack(Block.dirt)});
        }
    }

    public static void modsLoaded()
    {
        // Overwrite blocks
        Block.blocksList[Block.oreDiamond.blockID] = null;
        Block.blocksList[Block.oreDiamond.blockID] = new SpelunkerBlockOre(56).setSpelunkerScore(10000)
                .setObtainedSound("spelunker:diamond").setHardness(3.0F).setResistance(5.0F).setStepSound(Block.soundStoneFootstep)
                .setUnlocalizedName("oreDiamond").func_111022_d("diamond_ore");

        Block.blocksList[Block.oreEmerald.blockID] = null;
        Block.blocksList[Block.oreEmerald.blockID] = new SpelunkerBlockOre(129).setSpelunkerScore(8000)
                .setHardness(3.0F).setResistance(5.0F).setStepSound(Block.soundStoneFootstep)
                .setUnlocalizedName("oreEmerald").func_111022_d("emerald_ore");

        Block.blocksList[Block.oreLapis.blockID] = null;
        Block.blocksList[Block.oreLapis.blockID] = (new SpelunkerBlockOre(21).setSpelunkerScore(5000))
                .setHardness(3.0F).setResistance(5.0F).setStepSound(Block.soundStoneFootstep)
                .setUnlocalizedName("oreLapis").func_111022_d("lapis_ore");;

        Block.blocksList[Block.oreRedstone.blockID] = null;
        Block.blocksList[Block.oreRedstone.blockID] = (new SpelunkerBlockRedstoneOre(73, false)
                .setSpelunkerScore(1500)).setHardness(3.0F).setResistance(5.0F).setStepSound(Block.soundStoneFootstep)
                .setUnlocalizedName("oreRedstone").setCreativeTab(CreativeTabs.tabBlock).func_111022_d("redstone_ore");

        Block.blocksList[Block.oreRedstoneGlowing.blockID] = null;
        Block.blocksList[Block.oreRedstoneGlowing.blockID] = (new SpelunkerBlockRedstoneOre(74, true)
                .setSpelunkerScore(1500)).setLightValue(0.625F).setHardness(3.0F).setResistance(5.0F)
                .setStepSound(Block.soundStoneFootstep).setUnlocalizedName("oreRedstone").func_111022_d("redstone_ore");
    }

    public SpelunkerItem(int par1)
    {
        super(par1);
        dropItemRegistry.add(this);
    }

    @Override
    public Item setUnlocalizedName(String par1Str)
    {
        this.func_111206_d(par1Str);
        return super.setUnlocalizedName(par1Str);
    }

    public abstract void giveEffect(World world, SpelunkerPlayerMP spelunker);

    public int getRarity(SpelunkerDifficulty difficulty)
    {
        return 10;
    }

    public boolean spawnCheck(SpelunkerPlayerMP spelunker, SpelunkerDifficulty difficulty, boolean isDarkPlace, double posY)
    {
        return true;
    }

    public void animate() {}
}
