package tsuteto.spelunker.packet;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.logging.Level;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.network.packet.Packet250CustomPayload;
import tsuteto.spelunker.SpelunkerMod;
import tsuteto.spelunker.constants.SpelunkerPacketType;
import tsuteto.spelunker.util.ModLog;
import cpw.mods.fml.common.network.PacketDispatcher;
import cpw.mods.fml.common.network.Player;

/**
 * Simplifies the process to dispatch packets for Spelunker
 *
 * @author Tsuteto
 *
 */
public class SpelunkerPacketDispatcher
{
	private ByteArrayOutputStream bytesStream;
	private DataOutputStream dataStream;

	public SpelunkerPacketDispatcher(SpelunkerPacketType type)
	{
        bytesStream = new ByteArrayOutputStream();
        dataStream = new DataOutputStream(bytesStream);

        this.addByte((byte)(type.ordinal() & 0xff));
	}

	public SpelunkerPacketDispatcher addInt(int val)
	{
        try
        {
            dataStream.writeInt(val);
        }
        catch (IOException e)
        {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
        return this;
	}

    public SpelunkerPacketDispatcher addByte(byte val)
    {
        try
        {
            dataStream.writeByte(val);
        }
        catch (IOException e)
        {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
        return this;
    }

    public SpelunkerPacketDispatcher addBool(boolean val)
    {
        try
        {
            dataStream.writeBoolean(val);
        }
        catch (IOException e)
        {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
        return this;
    }
    public SpelunkerPacketDispatcher addShort(short val)
    {
        try
        {
            dataStream.writeShort(val);
        }
        catch (IOException e)
        {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
        return this;
    }

    public SpelunkerPacketDispatcher addDouble(double val)
    {
        try
        {
            dataStream.writeDouble(val);
        }
        catch (IOException e)
        {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
        return this;
    }

    public SpelunkerPacketDispatcher addFloat(float val)
    {
        try
        {
            dataStream.writeFloat(val);
        }
        catch (IOException e)
        {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
        return this;
    }

    public SpelunkerPacketDispatcher addString(String val)
    {
       byte[] bytes = val.getBytes();
       try
       {
            dataStream.write(bytes);
            dataStream.writeShort(bytes.length);
        }
       catch (IOException e)
       {
            ModLog.log(Level.WARNING, e, "Failed to set value to a packet.");
        }
       return this;
    }

    public void sendPacketPlayer(Player player)
    {
        try
        {
            if (((EntityPlayerMP)player).playerNetServerHandler != null)
            {
                PacketDispatcher.sendPacketToPlayer(
                        new Packet250CustomPayload(SpelunkerMod.modId, bytesStream.toByteArray()), player);
            }
        }
        finally
        {
            this.close();
        }
    }

    public void sendPacketPlayer(EntityPlayer player)
    {
        this.sendPacketPlayer((Player)player);
    }

    public void sendPacketAll()
    {
        try
        {
            PacketDispatcher.sendPacketToAllPlayers(
                    new Packet250CustomPayload(SpelunkerMod.modId, bytesStream.toByteArray()));
        }
        finally
        {
            this.close();
        }
    }

    public void sendPacketToServer()
    {
        try
        {
            PacketDispatcher.sendPacketToServer(
                    new Packet250CustomPayload(SpelunkerMod.modId, bytesStream.toByteArray()));
        }
        finally
        {
            this.close();
        }
    }

    private void close()
    {
        try
        {
            if (bytesStream != null) bytesStream.close();
        }
        catch (IOException e) {}
        try
        {
            if (dataStream != null) dataStream.close();
        }
        catch (IOException e) {}
    }

}
