package tsuteto.spelunker.sidedproxy;

import net.minecraft.client.entity.EntityPlayerSP;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraftforge.common.MinecraftForge;
import tsuteto.spelunker.SpelunkerMod;
import tsuteto.spelunker.entity.EntityBatDroppings;
import tsuteto.spelunker.entity.EntityFlash;
import tsuteto.spelunker.entity.EntityFlashBullet;
import tsuteto.spelunker.entity.EntityGunBullet;
import tsuteto.spelunker.entity.EntitySpelunkerItem;
import tsuteto.spelunker.entity.RenderBatDroppings;
import tsuteto.spelunker.entity.RenderFlash;
import tsuteto.spelunker.entity.RenderFlashBullet;
import tsuteto.spelunker.entity.RenderGunBullet;
import tsuteto.spelunker.entity.RenderSpelunkerItem;
import tsuteto.spelunker.eventhandler.ClientTickHandler;
import tsuteto.spelunker.gui.ScreenRenderer;
import tsuteto.spelunker.player.ISpelunkerPlayer;
import tsuteto.spelunker.player.SpelunkerPlayerSP;
import tsuteto.spelunker.sound.SoundHandler;
import api.player.client.ClientPlayerAPI;
import cpw.mods.fml.client.registry.RenderingRegistry;
import cpw.mods.fml.common.registry.TickRegistry;
import cpw.mods.fml.relauncher.Side;

public class ClientProxy implements ISidedProxy
{

    /**
     * Returns Spelunker instance by PlayerAPI
     *
     * @param player
     * @return
     */
    @Override
    public <T extends ISpelunkerPlayer> T getSpelunkerPlayer(EntityPlayer player)
    {
        if (player instanceof EntityPlayerSP)
        {
            return (T) ((EntityPlayerSP) player).getClientPlayerBase(SpelunkerMod.modId);
        }
        else if (player instanceof EntityPlayerMP)
        {
            return (T) ((EntityPlayerMP) player).getServerPlayerBase(SpelunkerMod.modId);
        }
        // LittleMaid or entities extending EntityPlayer
        return null;
    }

    @Override
    public void registerComponent(SpelunkerMod mod)
    {
        MinecraftForge.EVENT_BUS.register(new ScreenRenderer());
        TickRegistry.registerTickHandler(new ClientTickHandler(mod), Side.CLIENT);

        ClientPlayerAPI.register(SpelunkerMod.modId, SpelunkerPlayerSP.class);

        RenderingRegistry.registerEntityRenderingHandler(EntityGunBullet.class, new RenderGunBullet());
        RenderingRegistry.registerEntityRenderingHandler(EntityBatDroppings.class, new RenderBatDroppings());
        RenderingRegistry.registerEntityRenderingHandler(EntityFlash.class, new RenderFlash());
        RenderingRegistry.registerEntityRenderingHandler(EntityFlashBullet.class, new RenderFlashBullet());
        RenderingRegistry.registerEntityRenderingHandler(EntitySpelunkerItem.class, new RenderSpelunkerItem());
    }

    @Override
    public void registerTextureFile(String filepath)
    {
    }

    @Override
    public void installSoundFiles()
    {
        MinecraftForge.EVENT_BUS.register(new SoundHandler());
    }

//    @Override
//    public void checkBgmSoundFile()
//    {
//        // BGM file check
//        Minecraft mc = Minecraft.getMinecraft();
//        File soundDir = new File(ModSoundManager.getAssetsDir(), "sound/Spelunker");
//        File[] files;
//
//        if (soundDir.exists())
//        {
//            files = soundDir.listFiles(new SoundFileFilter("main_bgm"));
//            if (files.length != 0)
//                SpelunkerMod.isBgmMainAvailable = true;
//
//            files = soundDir.listFiles(new SoundFileFilter("2xscore_bgm"));
//            if (files.length != 0)
//                SpelunkerMod.isBgm2xScoreAvailable = true;
//
//            files = soundDir.listFiles(new SoundFileFilter("invincible_bgm"));
//            if (files.length != 0)
//                SpelunkerMod.isBgmInvincibleAvailable = true;
//
//            files = soundDir.listFiles(new SoundFileFilter("speedpotion_bgm"));
//            if (files.length != 0)
//                SpelunkerMod.isBgmSpeedPotionAvailable = true;
//        }
//    }
}
